package xhttp3.internal.cache;

import java.util.Date;
import java.util.concurrent.TimeUnit;
import xhttp3.CacheControl;
import xhttp3.Headers;
import xhttp3.Request;
import xhttp3.Response;
import xhttp3.internal.Internal;
import xhttp3.internal.http.HttpDate;
import xhttp3.internal.http.HttpHeaders;

public final class CacheStrategy {
   public final Request networkRequest;
   public final Response cacheResponse;

   private CacheStrategy(Request networkRequest, Response cacheResponse) {
      this.networkRequest = networkRequest;
      this.cacheResponse = cacheResponse;
   }

   public static boolean isCacheable(Response response, Request request) {
      switch (response.code()) {
         case 200:
         case 203:
         case 204:
         case 300:
         case 301:
         case 308:
         case 404:
         case 405:
         case 410:
         case 414:
         case 501:
            break;
         case 302:
         case 307:
            if (response.header("Expires") != null || response.cacheControl().maxAgeSeconds() != -1 || response.cacheControl().isPublic() || response.cacheControl().isPrivate()) {
               break;
            }
         default:
            return false;
      }

      return !response.cacheControl().noStore() && !request.cacheControl().noStore();
   }

   // $FF: synthetic method
   CacheStrategy(Request x0, Response x1, Object x2) {
      this(x0, x1);
   }

   public static class Factory {
      final long nowMillis;
      final Request request;
      final Response cacheResponse;
      private Date servedDate;
      private String servedDateString;
      private Date lastModified;
      private String lastModifiedString;
      private Date expires;
      private long sentRequestMillis;
      private long receivedResponseMillis;
      private String etag;
      private int ageSeconds = -1;

      public Factory(long nowMillis, Request request, Response cacheResponse) {
         this.nowMillis = nowMillis;
         this.request = request;
         this.cacheResponse = cacheResponse;
         if (cacheResponse != null) {
            this.sentRequestMillis = cacheResponse.sentRequestAtMillis();
            this.receivedResponseMillis = cacheResponse.receivedResponseAtMillis();
            Headers nowMillis1;
            Headers var10000 = nowMillis1 = cacheResponse.headers();
            int var2 = 0;

            for(int request1 = var10000.size(); var2 < request1; ++var2) {
               String cacheResponse1 = nowMillis1.name(var2);
               String var5 = nowMillis1.value(var2);
               if ("Date".equalsIgnoreCase(cacheResponse1)) {
                  this.servedDate = HttpDate.parse(var5);
                  this.servedDateString = var5;
               } else if ("Expires".equalsIgnoreCase(cacheResponse1)) {
                  this.expires = HttpDate.parse(var5);
               } else if ("Last-Modified".equalsIgnoreCase(cacheResponse1)) {
                  this.lastModified = HttpDate.parse(var5);
                  this.lastModifiedString = var5;
               } else if ("ETag".equalsIgnoreCase(cacheResponse1)) {
                  this.etag = var5;
               } else if ("Age".equalsIgnoreCase(cacheResponse1)) {
                  this.ageSeconds = HttpHeaders.parseSeconds(var5, -1);
               }
            }
         }

      }

      private CacheStrategy getCandidate() {
         if (this.cacheResponse == null) {
            return new CacheStrategy(this.request, (Response)null);
         } else if (this.request.isHttps() && this.cacheResponse.handshake() == null) {
            return new CacheStrategy(this.request, (Response)null);
         } else if (!CacheStrategy.isCacheable(this.cacheResponse, this.request)) {
            return new CacheStrategy(this.request, (Response)null);
         } else {
            CacheControl var1;
            if (!(var1 = this.request.cacheControl()).noCache() && !hasConditions(this.request)) {
               long var2 = this.cacheResponseAge();
               long var4 = this.computeFreshnessLifetime();
               if (var1.maxAgeSeconds() != -1) {
                  var4 = Math.min(var4, TimeUnit.SECONDS.toMillis((long)var1.maxAgeSeconds()));
               }

               long var6 = 0L;
               if (var1.minFreshSeconds() != -1) {
                  var6 = TimeUnit.SECONDS.toMillis((long)var1.minFreshSeconds());
               }

               long var8 = 0L;
               CacheControl var10;
               if (!(var10 = this.cacheResponse.cacheControl()).mustRevalidate() && var1.maxStaleSeconds() != -1) {
                  var8 = TimeUnit.SECONDS.toMillis((long)var1.maxStaleSeconds());
               }

               if (!var10.noCache() && (var6 += var2) < var4 + var8) {
                  Response.Builder var14 = this.cacheResponse.newBuilder();
                  if (var6 >= var4) {
                     var14.addHeader("Warning", "110 HttpURLConnection \"Response is stale\"");
                  }

                  if (var2 > 86400000L && this.isFreshnessLifetimeHeuristic()) {
                     var14.addHeader("Warning", "113 HttpURLConnection \"Heuristic expiration\"");
                  }

                  return new CacheStrategy((Request)null, var14.build());
               } else {
                  String var11;
                  String var12;
                  if ((var11 = this.etag) != null) {
                     var12 = "If-None-Match";
                  } else if (this.lastModified != null) {
                     var12 = "If-Modified-Since";
                     var11 = this.lastModifiedString;
                  } else {
                     if (this.servedDate == null) {
                        return new CacheStrategy(this.request, (Response)null);
                     }

                     var12 = "If-Modified-Since";
                     var11 = this.servedDateString;
                  }

                  Headers.Builder var3 = this.request.headers().newBuilder();
                  Internal.instance.addLenient(var3, var12, var11);
                  Request var13 = this.request.newBuilder().headers(var3.build()).build();
                  return new CacheStrategy(var13, this.cacheResponse);
               }
            } else {
               return new CacheStrategy(this.request, (Response)null);
            }
         }
      }

      private long computeFreshnessLifetime() {
         CacheControl var1;
         if ((var1 = this.cacheResponse.cacheControl()).maxAgeSeconds() != -1) {
            return TimeUnit.SECONDS.toMillis((long)var1.maxAgeSeconds());
         } else {
            Date var3;
            long var4;
            if (this.expires != null) {
               if ((var3 = this.servedDate) != null) {
                  var4 = var3.getTime();
               } else {
                  var4 = this.receivedResponseMillis;
               }

               if ((var4 = this.expires.getTime() - var4) <= 0L) {
                  var4 = 0L;
               }

               return var4;
            } else if (this.lastModified != null && this.cacheResponse.request().url().query() == null) {
               return (var4 = ((var3 = this.servedDate) != null ? var3.getTime() : this.sentRequestMillis) - this.lastModified.getTime()) > 0L ? var4 / 10L : 0L;
            } else {
               return 0L;
            }
         }
      }

      private long cacheResponseAge() {
         Date var1;
         long var4;
         if ((var1 = this.servedDate) != null) {
            var4 = this.receivedResponseMillis - var1.getTime();
            var4 = Math.max(0L, var4);
         } else {
            var4 = 0L;
         }

         int var3;
         if ((var3 = this.ageSeconds) != -1) {
            var4 = Math.max(var4, TimeUnit.SECONDS.toMillis((long)var3));
         }

         long var10000 = var4;
         long var10001 = (var4 = this.receivedResponseMillis) - this.sentRequestMillis;
         var4 = this.nowMillis - var4;
         return var10000 + var10001 + var4;
      }

      private boolean isFreshnessLifetimeHeuristic() {
         return this.cacheResponse.cacheControl().maxAgeSeconds() == -1 && this.expires == null;
      }

      private static boolean hasConditions(Request request) {
         return request.header("If-Modified-Since") != null || request.header("If-None-Match") != null;
      }

      public CacheStrategy get() {
         CacheStrategy var1;
         return (var1 = this.getCandidate()).networkRequest != null && this.request.cacheControl().onlyIfCached() ? new CacheStrategy((Request)null, (Response)null) : var1;
      }
   }
}
